/*
 * @brief This header file is part of Ding RTC engine sdk for iOS/Mac system
 *
 * @copyright Copyright (c) 2021, Alibaba Inc.
 * All rights reserved
 */

#import <TargetConditionals.h>

#if TARGET_OS_IPHONE
#import <UIKit/UIKit.h>
typedef UIColor DingRtcOSColor;
typedef UIImage DingRtcOSImage;
typedef UIView DingRtcOSView;
typedef UITextView DingRtcOSTextView;
#else
#import <AppKit/AppKit.h>
typedef NSColor DingRtcOSColor;
typedef NSImage DingRtcOSImage;
typedef NSView DingRtcOSView;
typedef NSTextView DingRtcOSTextView;
#endif
#import <CoreVideo/CoreVideo.h>

/**
* @defgroup OC_DingRtcDef 关键类型定义
* DingRtc 关键类型定义
*/

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频流类型。
 */
typedef NS_ENUM(NSUInteger, DingRtcVideoTrack) {
    /** DingRtcVideoTrackNo - 无摄像头和屏幕共享。 */
    DingRtcVideoTrackNo     = 0,
    /** DingRtcVideoTrackCamera - 摄像头。 */
    DingRtcVideoTrackCamera = 1,
    /** DingRtcVideoTrackScreen - 屏幕共享。 */
    DingRtcVideoTrackScreen = 2,
    /** DingRtcVideoTrackBoth - 摄像头+屏幕共享。 */
    DingRtcVideoTrackBoth   = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频流类型。
 */
typedef NS_ENUM(NSUInteger, DingRtcAudioTrack) {
    /** DingRtcAudioTrackNo - 无麦克风。 */
    DingRtcAudioTrackNo   = 0,
    /** DingRtcAudioTrackMic - 麦克风。 */
    DingRtcAudioTrackMic  = 1,
};


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 网络质量。
 */
typedef NS_ENUM(NSUInteger, DingRtcNetworkQuality) {
    /** DingRtcNetworkQualityGood - 网络质量好。 */
    DingRtcNetworkQualityGood       = 0,
    /** DingRtcNetworkQualityPoor - 网络质量差。 */
    DingRtcNetworkQualityPoor       = 1,
    /** DingRtcNetworkQualityDisconnect - 网络中断。 */
    DingRtcNetworkQualityDisconnect = 2,
    /** DingRtcNetworkQualityUnknown - 未知。 */
    DingRtcNetworkQualityUnknown     = 3,
};


/**
 * @brief 音频质量
*/
typedef NS_ENUM(NSInteger, DingRtcTranportAudioQuality) {
    /** DingRtcAudioQulityUnknow - 质量未知。 */
    DingRtcAudioQulityUnknow = 0,
    /** DingRtcAudioQulityExcellent - 质量极好。 */
    DingRtcAudioQulityExcellent = 1,
    /** DingRtcAudioQulityGood - 用户主观感觉和极好差不多 ，但码率可能略低于极好。 */
    DingRtcAudioQulityGood = 2,
    /** DingRtcAudioQulityPoor - 用户主观感受有瑕疵，但不影响沟通。 */
    DingRtcAudioQulityPoor = 3,
    /** DingRtcAudioQulityBad - 勉强能沟通但不顺畅。 */
    DingRtcAudioQulityBad = 4,
    /** DingRtcAudioQulityVeryBad - 网络质量非常差，基本不能沟通。 */
    DingRtcAudioQulityVeryBad = 5,
    /** DingRtcAudioQulityNetworkDisconnected - 网络连接已断开，完全无法沟通。 */
    DingRtcAudioQulityNetworkDisconnected = 6,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频编码模式。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioProfile) {
    /** DingRtcEngineBasicQualityMode - 标准音质模式，默认16000Hz采样率，单声道，最大编码码率24kbps。 */
    DingRtcEngineBasicQualityMode      = 0x0000,
    /** DingRtcEngineHighQualityMode - 高音质模式，默认48000Hz采样率，单声道，最大编码码率48kbps。 */
    DingRtcEngineHighQualityMode       = 0x0001,
    /** DingRtcEngineSuperHighQualityMode - 超高音质模式，默认48000Hz采样率，单声道，最大编码码率96kbps。 */
    DingRtcEngineSuperHighQualityMode = 0x0002,
    // /** 立体声高音质模式，默认48000Hz采样率，双声道，最大编码码率80kbps。 */
    // DingRtcEngineStereoHighQualityMode = 0x0003,
    // /** 立体声超高音质模式，默认48000Hz采样率，双声道，最大编码码率128kbps。 */
    // DingRtcEngineStereoSuperHighQualityMode = 0x0004,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频场景模式。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioScenario) {
    /** DingRtcSceneDefaultMode - 默认场景，一般的音视频通信场景推荐使用。 */
    DingRtcSceneDefaultMode   = 0x0000,
    /** DingRtcSceneMusicMode - 音乐场景，高音质场景，适用于音乐为主的场景。 */
    DingRtcSceneMusicMode     = 0x0001,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视图显示模式。
 */
typedef NS_ENUM(NSUInteger, DingRtcRenderMode) {
    /** DingRtcRenderModeAuto - 自动模式。 */
    DingRtcRenderModeAuto    = 0,
    /** DingRtcRenderModeStretch - 延伸模式。 */
    DingRtcRenderModeStretch = 1,
    /** DingRtcRenderModeFill - 填充模式。 */
    DingRtcRenderModeFill    = 2,
    /** DingRtcRenderModeCrop - 裁剪模式。 */
    DingRtcRenderModeCrop    = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief Log级别。
 */
typedef NS_ENUM(NSUInteger, DingRtcLogLevel) {
    /** DingRtcLogLevelInfo - 只输出>=DingRtcLogLevelInfo 级别的日志。 */
    DingRtcLogLevelInfo = 3,
    /** DingRtcLogLevelWarn - 只输出>=DingRtcLogLevelWarn 级别的日志。 */
    DingRtcLogLevelWarn = 4,
    /** DingRtcLogLevelError - 只输出>=DingRtcLogLevelError 级别的日志。 */
    DingRtcLogLevelError = 5,
    /** DingRtcLogLevelNone - 不输出日志。 */
    DingRtcLogLevelNone = 6,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频编码帧率。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoEncoderFrameRate) {
    /** DingRtcVideoEncoderFrameRate_FPS_5 - 5 FPS。 */
    DingRtcVideoEncoderFrameRate_FPS_5 = 5,
    /** DingRtcVideoEncoderFrameRate_FPS_10 - 10 FPS。 */
    DingRtcVideoEncoderFrameRate_FPS_10 = 10,
    /** DingRtcVideoEncoderFrameRate_FPS_15 - 15 FPS。 */
    DingRtcVideoEncoderFrameRate_FPS_15 = 15,
    /** DingRtcVideoEncoderFrameRate_FPS_20 - 20 FPS。 */
    DingRtcVideoEncoderFrameRate_FPS_20 = 20,
    /** DingRtcVideoEncoderFrameRate_FPS_30 - 30 FPS。 */
    DingRtcVideoEncoderFrameRate_FPS_30 = 30,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 编码视频镜像模式。

 - DingRtcVideoEncoderMirrorModeDisabled：编码视频镜像
 - DingRtcVideoEncoderMirrorModeEnabled：编码视频不镜像
 */
typedef NS_ENUM(NSInteger, DingRtcVideoEncoderMirrorMode) {
    /** DingRtcVideoEncoderMirrorModeDisabled - 编码视频镜像。 */
    DingRtcVideoEncoderMirrorModeDisabled = 0,
    /** DingRtcVideoEncoderMirrorModeEnabled - 编码视频不镜像。 */
    DingRtcVideoEncoderMirrorModeEnabled = 1,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频旋转角度。
 */
typedef NS_ENUM(NSInteger, DingRtcRotationMode) {
    /** DingRtcRotationMode_0 - 旋转0度。 */
  DingRtcRotationMode_0    = 0,
    /** DingRtcRotationMode_90 - 旋转90度。 */
  DingRtcRotationMode_90   = 90,
    /** DingRtcRotationMode_180 - 旋转180度。 */
  DingRtcRotationMode_180  = 180,
    /** DingRtcRotationMode_270 - 旋转270度。 */
  DingRtcRotationMode_270  = 270,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 编码视频旋转模式。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoEncoderOrientationMode) {
    /** DingRtcVideoEncoderOrientationModeAdaptive - 自适应，和采集视频保持一致。 */
    DingRtcVideoEncoderOrientationModeAdaptive = 0,
    /** DingRtcVideoEncoderOrientationModeFixedLandscape - 固定横屏。 */
    DingRtcVideoEncoderOrientationModeFixedLandscape = 1,
    /** DingRtcVideoEncoderOrientationModeFixedPortrait - 固定竖屏。 */
    DingRtcVideoEncoderOrientationModeFixedPortrait = 2,
};

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief 设备方向 （仅适用于 iOS 系统）。
 */
typedef NS_ENUM(NSInteger, DingRtcOrientationMode) {
    /** DingRtcOrientationModePortrait - 竖屏模式。 */
    DingRtcOrientationModePortrait = 0,
    /** DingRtcOrientationModeLandscapeLeft - 左横屏模式。 */
    DingRtcOrientationModeLandscapeLeft = 90,
    /** DingRtcOrientationModePortraitUpsideDown - 倒立模式。 */
    DingRtcOrientationModePortraitUpsideDown = 180,
    /** DingRtcOrientationModeLandscapeRight - 右横屏模式。 */
    DingRtcOrientationModeLandscapeRight = 270,
    /** DingRtcOrientationModeAuto - 由SDK自己决定。 */
    DingRtcOrientationModeAuto = 4,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief SDK对Audio Session的控制权限 （仅适用于 iOS 系统）。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioSessionOperationRestriction) {
    /** DingRtcAudioSessionOperationRestrictionNone - 没有限制，SDK完全控制AVAudioSession。 */
    DingRtcAudioSessionOperationRestrictionNone              = 0,
    /** DingRtcAudioSessionOperationRestrictionSetCategory - SDK不能修改AVAudioSession的Category。 */
    DingRtcAudioSessionOperationRestrictionSetCategory       = 1 << 0,
    /** DingRtcAudioSessionOperationRestrictionConfigureSession - SDK不能修改AVAudioSession的配置，包括Category，Mode，CategoryOptions。 */
    DingRtcAudioSessionOperationRestrictionConfigureSession  = 1 << 1,
    /** DingRtcAudioSessionOperationRestrictionDeactivateSession - SDK不能关闭AVAudioSession的活动状态，离开频道时，AVAudioSession依然处于活动状态。 */
    DingRtcAudioSessionOperationRestrictionDeactivateSession = 1 << 2,
};
#endif


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 相机流类型。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoStreamType) {
    /** DingRtcVideoStreamTypeNone - 无，在OnSubscribeStreamTypeChanged回调表示当前未订阅。 */
    DingRtcVideoStreamTypeNone = 0,
    /** DingRtcVideoStreamTypeFHD - 超高清流（设备性能、网络状态、推流分辨率符合要求情况下满足），对应1080P收费档。 */
    DingRtcVideoStreamTypeFHD = 1,
    /** DingRtcVideoStreamTypeHD - 高清流（设备性能、网络状态、推流分辨率符合要求情况下满足），对应720P收费档。 */
    DingRtcVideoStreamTypeHD = 2,
    /** DingRtcVideoStreamTypeSD - 标清（设备性能、网络状态、推流分辨率符合要求情况下满足），对应480P收费档。 */
    DingRtcVideoStreamTypeSD = 3,
    /** DingRtcVideoStreamTypeLD - 流畅（设备性能、网络状态、推流分辨率符合要求情况下满足），对应480P收费档。 */
    DingRtcVideoStreamTypeLD = 4,
};

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief 设备音量类型 （仅适用于 iOS 系统）。
 * @note 默认为Auto类型，SDK 内部会自动选择合适的音量类型，如无特殊需求，不推荐自行设置。
 */
typedef NS_ENUM(NSInteger, DingRtcDeviceVolumeType) {
    /** DingRtcDeviceVolumeTypeAuto - SDK自动设置音量类型。 */
    DingRtcDeviceVolumeTypeAuto = 0,
    /** DingRtcDeviceVolumeTypeMedia - 设置为媒体音量，适合媒体播放场景，音量可以调节到零。 */
    DingRtcDeviceVolumeTypeMedia = 1,
    /** DingRtcDeviceVolumeTypeVOIP - 设置为通话音量，适合通话场景，音量无法调节到零。 */
    DingRtcDeviceVolumeTypeVOIP = 2,
};
#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频裸数据源类型。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoSource) {
    /** DingRtcVideosourceCameraType - 视频流。 */
    DingRtcVideosourceCameraType = 0,
    /** DingRtcVideosourceScreenShareType - 屏幕共享流。 */
    DingRtcVideosourceScreenShareType = 1,
#if TARGET_OS_MAC && !TARGET_OS_IPHONE
    /** DingRtcVideosourceBoth - 视频流和屏幕共享流（仅适用于 Mac 系统）。 */
    DingRtcVideosourceBoth = 2,
#endif
    /** DingRtcVideosourceTypeMax - 占位符，无意义。 */
    DingRtcVideosourceTypeMax,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.1
 * @brief 音频数据源类型。
 */
typedef NS_OPTIONS(NSInteger, DingRtcAudioObservePosition) {
    /*! DingRtcAudioObservePositionCaptured - 采集的音频数据。 */
    DingRtcAudioObservePositionCaptured = 1 << 0,
    /*! DingRtcAudioObservePositionProcessCaptured - 3A后的音频数据。 */
    DingRtcAudioObservePositionProcessCaptured = 1 << 1,
    /*! DingRtcAudioObservePositionPub - 推流的音频数据。 */
    DingRtcAudioObservePositionPub = 1 << 2,
    /*! DingRtcAudioObservePositionPlayback - 播放的音频数据。 */
    DingRtcAudioObservePositionPlayback = 1 << 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频数据源类型。
 * @deprecated 该类型已废弃，请使用 DingRtcAudioObservePosition 代替。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioSource) {
    /*! DingRtcAudioSourceCaptured - 采集的音频数据。 */
    DingRtcAudioSourceCaptured = 0,
    /*! DingRtcAudioSourceProcessCaptured - 3A后的音频数据。 */
    DingRtcAudioSourceProcessCaptured = 1,
    /*! DingRtcAudioSourcePub - 推流的音频数据。 */
    DingRtcAudioSourcePub = 2,
    /*! DingRtcAudioSourcePlayback - 播放的音频数据。 */
    DingRtcAudioSourcePlayback = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频数据格式。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoFormat) {
    /*! DingRtcVideoFormat_UNKNOW - 未知。 */
    DingRtcVideoFormat_UNKNOW = -1,
    /*! DingRtcVideoFormat_BGRA - BGRA。 */
    DingRtcVideoFormat_BGRA = 0,
    /*! DingRtcVideoFormat_I420 - I420。 */
    DingRtcVideoFormat_I420,
    /*! DingRtcVideoFormat_NV21 - NV21。 */
    DingRtcVideoFormat_NV21,
    /*! DingRtcVideoFormat_NV12 - NV12。 */
    DingRtcVideoFormat_NV12,
    /*! DingRtcVideoFormat_RGBA - RGBA。 */
    DingRtcVideoFormat_RGBA,
    /*! DingRtcVideoFormat_I422 - I422。 */
    DingRtcVideoFormat_I422,
    /*! DingRtcVideoFormat_ARGB - ARGB。 */
    DingRtcVideoFormat_ARGB,
    /*! DingRtcVideoFormat_ABGR - ABGR。 */
    DingRtcVideoFormat_ABGR,
    /*! DingRtcVideoFormat_RGB24 - RGB24。 */
    DingRtcVideoFormat_RGB24,
    /*! DingRtcVideoFormat_BGR24 - BGR24。 */
    DingRtcVideoFormat_BGR24,
    /*! DingRtcVideoFormat_RGB565 - RGB565。 */
    DingRtcVideoFormat_RGB565,
    /*! DingRtcVideoFormat_H264 - H264。 */
    DingRtcVideoFormat_H264,
};

typedef NS_OPTIONS(NSInteger, DingRtcVideoObservePosition) {
    /** 采集视频数据，对应输出回调 onCaptureVideoFrame */
    DingRtcVideoObservePositionPostCapture = 1 << 0,
    /** 渲染视频数据，对应输出回调 onRemoteVideoFrame */
    DingRtcVideoObservePositionPreRender   = 1 << 1,
    /** 编码前视频数据，对应输出回调 onPreEncodeVideoFrame */
    DingRtcVideoObservePositionPreEncoder  = 1 << 2,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频数据类型。

 - DingRtcBufferType_Raw_Data：YUV裸数据
 - DingRtcBufferType_CVPixelBuffer：CVPixelBuffer数据
 */
typedef NS_ENUM(NSInteger, DingRtcBufferType) {
    /** DingRtcBufferType_Raw_Data - YUV裸数据。 */
    DingRtcBufferType_Raw_Data = 0,
    /** DingRtcBufferType_CVPixelBuffer - CVPixelBuffer数据。 */
    DingRtcBufferType_CVPixelBuffer,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频声道数。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioNumChannel) {
    /** DingRtcMonoAudio - 单声道。 */
    DingRtcMonoAudio = 1,
    /** DingRtcStereoAudio - 双声道。 */
    DingRtcStereoAudio = 2,
};

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief 文件录制质量 （仅适用于 iOS 系统）。
 */
typedef NS_ENUM(NSInteger, DingRtcRecordingQuality) {
    /** DingRtcRecording_Low_Quality - 低质量文件录制，文件体积较小，音质一般。 */
    DingRtcRecording_Low_Quality = 0,
    /** DingRtcRecording_Midium_Quality - 中等质量文件录制，文件体积中等，音质中等。 */
    DingRtcRecording_Midium_Quality = 1,
    /** DingRtcRecording_High_Quality - 高质量文件录制，文件体积较大，音质较好。 */
    DingRtcRecording_High_Quality = 2,
};
#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 变声音效模式。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioEffectVoiceChangerMode) {
    /** DingRtcAudioEffectvVoiceChanger_OFF - 关闭变声音效。 */
    DingRtcAudioEffectvVoiceChanger_OFF = 0,
    /** DingRtcAudioEffectvVoiceChanger_OLD_MAN - 老人。 */
    DingRtcAudioEffectvVoiceChanger_OLD_MAN,
    /** DingRtcAudioEffectvVoiceChanger_BABYBOY - 男孩。 */
    DingRtcAudioEffectvVoiceChanger_BABYBOY,
    /** DingRtcAudioEffectvVoiceChanger_BABYGILR - 女孩。 */
    DingRtcAudioEffectvVoiceChanger_BABYGILR,
    /** DingRtcAudioEffectvVoiceChanger_ROBOT - 机器人。 */
    DingRtcAudioEffectvVoiceChanger_ROBOT,
    /** DingRtcAudioEffectvVoiceChanger_DAIMO - 大魔王。 */
    DingRtcAudioEffectvVoiceChanger_DAIMO,
    /** DingRtcAudioEffectvVoiceChanger_KTV - KTV。 */
    DingRtcAudioEffectvVoiceChanger_KTV,
    /** DingRtcAudioEffectvVoiceChangerR_ECHO - 回声。 */
    DingRtcAudioEffectvVoiceChangerR_ECHO,
    /** DingRtcAudioEffectvVoiceChanger_MAX - 占位符。 */
    DingRtcAudioEffectvVoiceChanger_MAX,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音效混响模式。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioEffectReverbMode) {
    /** DingRtcAudioEffectReverb_Off - 关闭混响。 */
    DingRtcAudioEffectReverb_Off = 0,
    /** DingRtcAudioEffectReverb_Vocal_I - 人声I。 */
    DingRtcAudioEffectReverb_Vocal_I,
    /** DingRtcAudioEffectReverb_Vocal_II - 人声II。 */
    DingRtcAudioEffectReverb_Vocal_II,
    /** DingRtcAudioEffectReverb_Bathroom - 澡堂。 */
    DingRtcAudioEffectReverb_Bathroom,
    /** DingRtcAudioEffectReverb_Small_Room_Bright - 明亮小房间。 */
    DingRtcAudioEffectReverb_Small_Room_Bright,
    /** DingRtcAudioEffectReverb_Small_Room_Dark - 黑暗小房间。 */
    DingRtcAudioEffectReverb_Small_Room_Dark,
    /** DingRtcAudioEffectReverb_Medium_Room - 中等房间。 */
    DingRtcAudioEffectReverb_Medium_Room,
    /** DingRtcAudioEffectReverb_Large_Room - 大房间。 */
    DingRtcAudioEffectReverb_Large_Room,
    /** DingRtcAudioEffectReverb_Church_Hall - 教堂走廊。 */
    DingRtcAudioEffectReverb_Church_Hall,
    /** DingRtcAudioEffectReverb_Mode_Max - 占位符。 */
    DingRtcAudioEffectReverb_Mode_Max,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音效混响参数。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioEffectReverbParamType) {
    /** DingRtcAudioEffectReverb_Room_Size - 房间大小，取值范围：[0, 100]。 */
    DingRtcAudioEffectReverb_Room_Size = 0,
    /** DingRtcAudioEffectReverb_Pre_Delay - 预延时，单位ms，取值范围：[0, 200]。 */
    DingRtcAudioEffectReverb_Pre_Delay,
    /** DingRtcAudioEffectReverb_Revetberance - 混响感，取值范围：[0, 100]。 */
    DingRtcAudioEffectReverb_Revetberance,
    /** DingRtcAudioEffectReverb_Hf_Damping - 消声，取值范围：[0, 100]。 */
    DingRtcAudioEffectReverb_Hf_Damping,
    /** DingRtcAudioEffectReverb_Tone_Low - 低音调，取值范围：[0, 100]。 */
    DingRtcAudioEffectReverb_Tone_Low,
    /** DingRtcAudioEffectReverb_Tone_High - 高音调，取值范围：[0, 100]。 */
    DingRtcAudioEffectReverb_Tone_High,
    /** DingRtcAudioEffectReverb_Dry_Gain - 干增益，取值范围：[-20, 10]。 */
    DingRtcAudioEffectReverb_Dry_Gain,
    /** DingRtcAudioEffectReverb_Wet_Gain - 湿增益，取值范围：[-20, 10]。 */
    DingRtcAudioEffectReverb_Wet_Gain,
    /** DingRtcAudioEffectReverb_Type_Max - 占位符。 */
    DingRtcAudioEffectReverb_Type_Max,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 网络连接状态。
 */
typedef NS_ENUM(NSInteger, DingRtcConnectionStatus) {
    /** DingRtcConnectionStatusInit - 初始化完成。 */
    DingRtcConnectionStatusInit = 0,
    /** DingRtcConnectionStatusDisconnected - 网络连接断开。 */
    DingRtcConnectionStatusDisconnected = 1,
    /** DingRtcConnectionStatusConnecting - 建立网络连接中。 */
    DingRtcConnectionStatusConnecting = 2,
    /** DingRtcConnectionStatusConnected - 网络已连接。 */
    DingRtcConnectionStatusConnected = 3,
    /** DingRtcConnectionStatusReconnecting - 重新建立网络连接中。 */
    DingRtcConnectionStatusReconnecting = 4,
    /** DingRtcConnectionStatusFailed - 网络连接失败。 */
    DingRtcConnectionStatusFailed = 5,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 引起网络状态变更原因。
 */
typedef NS_ENUM(NSInteger, DingRtcConnectionStatusChangeReason) {
    /** DingRtcConnectionChangedDummyReason - 未知原因。 */
    DingRtcConnectionChangedDummyReason = 0,
    /** DingRtcConnectionChangedSignalingHeartbeatTimeout - 心跳超时。 */
    DingRtcConnectionChangedSignalingHeartbeatTimeout = 1,
    /** DingRtcConnectionChangedSignalingHeartbeatAlive - 心跳恢复。 */
    DingRtcConnectionChangedSignalingHeartbeatAlive = 2,
    /** DingRtcConnectionChangedSignalingJoinChannelFailure - 加入频道失败。 */
    DingRtcConnectionChangedSignalingJoinChannelFailure = 3,
    /** DingRtcConnectionChangedSignalingJoinChannelSuccess - 加入频道成功。 */
    DingRtcConnectionChangedSignalingJoinChannelSuccess = 4,
    /** DingRtcConnectionChangedSignalingLeaveRoom - 离开频道。 */
    DingRtcConnectionChangedSignalingLeaveRoom = 5,
    /** DingRtcConnectionChangedSignalingConnecting - 信令建立连接。 */
    DingRtcConnectionChangedSignalingConnecting = 6,
    /** DingRtcConnectionChangedMediaLinkChange - 媒体链路变化 */
    DingRtcConnectionChangedMediaLinkChange = 7,
    /** DingRtcConnectionChangedNetworkInterrupted - 网络连接中断。 */
    DingRtcConnectionChangedNetworkInterrupted = 8,
    /** DingRtcConnectionChangedNetworkRecovery - 网络连接恢复。 */
    DingRtcConnectionChangedNetworkRecovery = 9,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 频道类型。
 */
typedef NS_ENUM(NSInteger, DingRtcChannelProfile) {
    /** DingRtcCommunication - 通信模式。 */
    DingRtcCommunication = 0,
    /** DingRtcInteractivelive - 互动模式。 */
    DingRtcInteractivelive,
    /** DingRtcInteractiveWithLowLatencyLive - 低延时互动直播模式。 */
    DingRtcInteractiveWithLowLatencyLive,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 角色类型。
 */
typedef NS_ENUM(NSInteger, DingRtcClientRole) {
    /** DingRtcClientRoleInteractive - 互动角色。 */
    DingRtcClientRoleInteractive = 0,
    /** DingRtcClientRolelive - 观众角色。 */
    DingRtcClientRolelive,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 本地静音模式。
*/
typedef NS_ENUM(NSInteger, DingRtcMuteLocalAudioMode) {
    /** DingRtcMuteAudioModeDefault - 默认模式（静音全部，包括麦克风及外部输入音频）。 */
    DingRtcMuteAudioModeDefault = 0,
    /** DingRtcMuteAllAudioMode - 静音全部（包括麦克风及外部输入音频）。 */
    DingRtcMuteAllAudioMode,
    /** DingRtcMuteOnlyMicAudioMode - 只静音麦克风。 */
    DingRtcMuteOnlyMicAudioMode,
    DingRtcMuteLocalAudioMax,
};


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 镜像模式。
 */
typedef NS_ENUM(NSInteger, DingRtcRenderMirrorMode) {
    /** DingRtcRenderMirrorModeOnlyFrontCameraPreviewEnabled - 只有前置摄像头预览镜像，其余不镜像。 */
    DingRtcRenderMirrorModeOnlyFrontCameraPreviewEnabled = 0,
    /** DingRtcRenderMirrorModeAllEnabled - 镜像开启。 */
    DingRtcRenderMirrorModeAllEnabled = 1,
    /** DingRtcRenderMirrorModeAllDisabled - 镜像关闭。 */
    DingRtcRenderMirrorModeAllDisabled = 2,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.4
 * @brief 音乐伴奏混音状态。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioMixingStatus) {
    /** DingRtcAudioMixingNone - 初始状态。 */
    DingRtcAudioMixingNone          = 0,
    /** DingRtcAudioMixingStarted - 开始混音。 */
    DingRtcAudioMixingStarted       = 100,
    /** DingRtcAudioMixingStopped - 停止混音。 */
    DingRtcAudioMixingStopped       = 101,
    /** DingRtcAudioMixingPaused - 暂停混音。 */
    DingRtcAudioMixingPaused        = 102,
    /** DingRtcAudioMixingResumed - 恢复混音。 */
    DingRtcAudioMixingResumed       = 103,
    /** DingRtcAudioMixingEnded - 混音完毕。 */
    DingRtcAudioMixingEnded         = 104,
    /** DingRtcAudioMixingBuffering - 正在缓冲。 */
    DingRtcAudioMixingBuffering     = 105,
    /** DingRtcAudioMixingBufferingEnd - 缓冲结束。 */
    DingRtcAudioMixingBufferingEnd  = 106,
    /** DingRtcAudioMixingFailed - 混音失败。 */
    DingRtcAudioMixingFailed        = 107,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.4
 * @brief 音乐伴奏混音错误码。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioMixingErrorCode) {
    /** DingRtcAudioMixingNoError - 没有错误。 */
    DingRtcAudioMixingNoError        = 0,
    /** DingRtcAudioMixingOpenFailed - 打开文件失败。 */
    DingRtcAudioMixingOpenFailed     = -100,
    /** DingRtcAudioMixingDecodeFailed - 解码文件失败。 */
    DingRtcAudioMixingDecodeFailed   = -101,
};

/**
 * @brief 录制类型。
*/
typedef NS_ENUM(NSUInteger, DingRtcRecordType) {
    /** DingRtcRecordTypeAudio - 纯音频录制。 */
    DingRtcRecordTypeAudio = 0,
#if TARGET_OS_MAC && !TARGET_OS_IPHONE
    /** DingRtcRecordTypeVideo-录制单个音视频文件 （仅适用于 Mac 系统）。 */
    DingRtcRecordTypeVideo,
    /** DingRtcRecordTypeBoth-录制纯音频和音视频文件 （仅适用于 Mac 系统）。 */
    DingRtcRecordTypeBoth,
#endif
};

/**
 * @brief 录制格式。
 */
typedef NS_ENUM(NSUInteger, DingRtcRecordFormat) {
    /** DingRtcRecordFormatAAC - 录制AAC格式文件。 */
    DingRtcRecordFormatAAC = 0,
    /** DingRtcRecordFormatWAV - 录制WAV格式文件。 */
    DingRtcRecordFormatWAV,
#if TARGET_OS_MAC && !TARGET_OS_IPHONE
    /** DingRtcRecordFormatMP4 - 录制MP4格式文件（仅适用于 Mac 系统）。 */
    DingRtcRecordFormatMP4,
#endif
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制音频质量。
 */
typedef NS_ENUM(NSUInteger, DingRtcAudioQuality) {
    /** DingRtcAudioQualityLow - 低质量文件录制，文件体积较小，音质一般。 */
    DingRtcAudioQualityLow = 0,
    /** DingRtcAudioQualityMidium - 中等质量文件录制，文件体积中等，音质中等。 */
    DingRtcAudioQualityMidium,
    /** DingRtcAudioQualityHigh - 高质量文件录制，文件体积较大，音质较好。 */
    DingRtcAudioQualityHigh,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制视频质量。
 */
typedef NS_ENUM(NSUInteger, DingRtcVideoQuality) {
    /**  DingRtcVideoQualityDefault - 默认，和采集保持一致。 */
    DingRtcVideoQualityDefault = 0,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制音频采样率。
*/
typedef NS_ENUM(NSInteger, DingRtcAudioSampleRate) {
    /** DingRtcAudioSampleRate_8000 - 采样率：8000。 */
    DingRtcAudioSampleRate_8000 = 0,
    /** DingRtcAudioSampleRate_11025 - 采样率：11025。 */
    DingRtcAudioSampleRate_11025 = 1,
    /** DingRtcAudioSampleRate_16000 - 采样率：16000。 */
    DingRtcAudioSampleRate_16000 = 2,
    /** DingRtcAudioSampleRate_22050 - 采样率：22050。 */
    DingRtcAudioSampleRate_22050 = 3,
    /** DingRtcAudioSampleRate_32000 - 采样率：32000。 */
    DingRtcAudioSampleRate_32000 = 4,
    /** DingRtcAudioSampleRate_44100 - 采样率：44100。 */
    DingRtcAudioSampleRate_44100 = 5,
    /** DingRtcAudioSampleRate_48000 - 采样率：48000。 */
    DingRtcAudioSampleRate_48000 = 6,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.3
 * @brief 音频数据源权限。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioFramePermission) {
    /*! DingRtcAudioFrameReadOnly - 只读权限。 */
    DingRtcAudioFrameReadOnly = 0,
    /*! DingRtcAudioFrameReadAndWrite - 读写权限 */
    DingRtcAudioFrameReadAndWrite = 1,
};

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief 语音路由 （仅适用于 iOS 系统）。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioRouteType) {
    /** DingRtcAudioRouteTypeDefault - 默认的语音路由。 */
    DingRtcAudioRouteTypeDefault        = 0,
    /** DingRtcAudioRouteTypeHeadset - 耳机。 */
    DingRtcAudioRouteTypeHeadset        = 1,
    /** DingRtcAudioRouteTypeEarpiece - 听筒。 */
    DingRtcAudioRouteTypeEarpiece       = 2,
    /** DingRtcAudioRouteTypeHeadsetNoMic - 不带麦克风的耳机。 */
    DingRtcAudioRouteTypeHeadsetNoMic   = 3,
    /** DingRtcAudioRouteTypeSpeakerphone - 手机扬声器。 */
    DingRtcAudioRouteTypeSpeakerphone   = 4,
    /** DingRtcAudioRouteTypeLoudSpeaker - 外接扬声器。 */
    DingRtcAudioRouteTypeLoudSpeaker    = 5,
    /** DingRtcAudioRouteTypeBlueTooth - 蓝牙耳机。 */
    DingRtcAudioRouteTypeBlueTooth      = 6,
};
#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 显示仪表盘类型。
 */
typedef NS_ENUM(NSInteger, DingRtcShowDebugViewType){
    /** DingRtcShowTypeNone - 不显示。 */
    DingRtcShowTypeNone     = 0,
    /** DingRtcShowTypeAudio - 音频。 */
    DingRtcShowTypeAudio    = 1,
    /** DingRtcShowTypeVideo - 视频。 */
    DingRtcShowTypeVideo    = 2,
    /** DingRtcShowTypeNetwork - 网络。 */
    DingRtcShowTypeNetwork  = 3,
    /** DingRtcShowTypeAll - 全部。 */
    DingRtcShowTypeAll      = 4,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 屏幕共享模式 支持位运算按需组合。
 */
typedef NS_ENUM(NSInteger, DingRtcScreenShareMode){
    /** DingRtcScreenShareNone - 不共享音视频流。 */
    DingRtcScreenShareNone       = 0,
    /** DingRtcScreenShareVideo - 共享视频流。 */
    DingRtcScreenShareVideo      = 1 << 0,
    /** DingRtcScreenShareAudioApp - 共享App音频流。 */
    DingRtcScreenShareAudioApp   = 1 << 1,
    /** DingRtcScreenShareAll - 共享所有音视频流，不支持屏幕共享Mic音频流，请使用SDK自带的Mic音频采集。 */
    DingRtcScreenShareAll        = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制音频设置。
 */
typedef struct  {
    /** 音频采样率类型。 */
    DingRtcAudioSampleRate sampleRate;
    /** 音频录制质量。 */
    DingRtcAudioQuality quality;
    /** 是否允许录制外部pcm输入渲染数据。 */
    bool enableRecordExternalRenderPCM;
#if TARGET_OS_MAC && !TARGET_OS_IPHONE
    /** 是否允许录制外部pcm输入采集数据  （仅适用于 Mac 系统）。 */
    bool enableRecordExternalCapturePCM;
#endif
}DingRtcRecordAudioConfig;

#if TARGET_OS_MAC && !TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制视频分辨率 （仅适用于 Mac 系统）。
 * @note 画布大小。
 */
typedef struct {
    /** 录制视频宽。 */
    int canvasWidth;
    /** 录制视频高。 */
    int canvasHeight;
} DingRtcRecordVideoCanvasConfig;

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制视频坐标 （仅适用于 Mac 系统）。
 */
typedef struct {
    uint32_t numerator;
    uint32_t denominator;
} DingRtcRecordVideoRational;

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制视频位置信息 （仅适用于 Mac 系统）。
 */
typedef struct {
    DingRtcRecordVideoRational left;
    DingRtcRecordVideoRational top;
    DingRtcRecordVideoRational width;
    DingRtcRecordVideoRational height;
} DingRtcRecordVideoRectangle;

#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 录制视频设置。
 */
typedef struct  {
    /** 视频质量。 */
    DingRtcVideoQuality quality;
#if TARGET_OS_MAC && !TARGET_OS_IPHONE
    /** 录制文件类型 （仅适用于 Mac 系统）。 */
    DingRtcRecordType sourceType;
    /** 录制文件尺寸 （仅适用于 Mac 系统）。 */
    DingRtcRecordVideoCanvasConfig canvas;
    /** 录制帧率 （仅适用于 Mac 系统）。 */
    int fps;
    /** 录制码率 （仅适用于 Mac 系统）。 */
    int bitrate;
#endif
} DingRtcRecordVideoConfig;


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 错误码。
 */
typedef NS_ENUM(NSInteger, DingRtcErrorCode) {
    /** DingRtcErrNone - 状态正常，无错误。 */
    DingRtcErrNone                           = 0,
    /** DingRtcErrJoinBadAppId - AppId不存在，请在控制台创建应用。 */
    DingRtcErrJoinBadAppId                   = 0x02010201,
    /** DingRtcErrJoinInvalidAppId - AppId已失效，请在控制台重新启用应用。 */
    DingRtcErrJoinInvalidAppId               = 0x02010202,
    /**
     * DingRtcErrJoinBadChannel - 频道不存在，请尝试重新加入频道。如会议发起者入会后又很快离会，第二个人随后入会，发现会议已经不存在了。
     */
    DingRtcErrJoinBadChannel                 = 0x02010204,
    /** DingRtcErrJoinInvalidChannel - 频道已经失效（目前频道有效期为48小时），请重新生成频道鉴权令牌（Token）。 */
    DingRtcErrJoinInvalidChannel             = 0x02010203,
    /** DingRtcErrJoinBadToken - Token无效。请重新生成频道鉴权令牌（Token）。 */
    DingRtcErrJoinBadToken                   = 0x02010205,
    /** DingRtcErrJoinTimeout - 加入频道超时，请检查网络连接是否正常。 */
    DingRtcErrJoinTimeout                    = 0x01020204,
    /** DingRtcErrJoinBadParam - 参数错误，{@link DingRtcAuthInfo}中的字段为空或者timestamp<=0，请检查字段是否完整。 */
    DingRtcErrJoinBadParam                   = 0x01030101,
    /** DingRtcErrJoinChannelFailed - 加入频道失败。 */
    DingRtcErrJoinChannelFailed              = 0x01030202,
    /** DingRtcErrLogAlreadyInit - 日志模块已经初始化。 */
    DingRtcErrLogAlreadyInit = 0x01030105,
    /** DingRtcErrLogRootPathError - 日志模块根路径错误，请检查路径是否存在或路径是否具有写入权限。 */
    DingRtcErrLogRootPathError = 0x01030106,
    /**
     * DingRtcErrMicOpenFail - 采集设备初始化失败、采集设备媒体服务异常、音频设备被占用。Mac端和Windows端查看系统设置中音频设备是否有能量条。iOS端和Android端确认是否被其他应用占用。如果都正常，请重启设备。
     */
    DingRtcErrMicOpenFail                    = 0x01040404,
    /**
     * DingRtcErrSpeakerOpenFail - 播放设备初始化失败、播放设备媒体服务异常、音频设备被占用。Mac端和Windows端查看系统设置中音频设备是否有能量条。iOS端和Android端确认是否被其他应用占用。如果都正常，请重启设备。
     */
    DingRtcErrSpeakerOpenFail                = 0x01040405,
    /** DingRtcErrMicInterrupt - 系统性能不足、采集设备媒体服务异常、音频设备被占用。建议重启应用或重启设备。 */
    DingRtcErrMicInterrupt                   = 0x01040406,
    /** DingRtcErrSpeakerInterrupt - 系统性能不足，播放设备媒体服务异常，音频设备被占用。建议重启应用或重启设备。 */
    DingRtcErrSpeakerInterrupt               = 0x01040407,
    /** DingRtcErrMicAuthFail - 麦克风设备未授权，建议去系统设置中，打开麦克风授权，并且重新加入频道。 */
    DingRtcErrMicAuthFail                    = 0x01040408,
    /** DingRtcErrMicNotAvailable - 无可用的音频采集设备，检查是否接入了可用的麦克风，并检查系统设置中，该设备是否被禁用。 */
    DingRtcErrMicNotAvailable                = 0x01040409,
    /** DingRtcErrSpeakerNotAvailable - 无可用的音频播放设备，检查是否接入了可用的扬声器或耳机，并检查系统设置中，该设备是否被禁用。 */
    DingRtcErrSpeakerNotAvailable            = 0x01040410,
    /** DingRtcErrCameraOpenFail - 采集设备初始化失败，检查系统相机是否可用，移动端App是否占用了相机。 */
    DingRtcErrCameraOpenFail                 = 0x01040104,
    /** DingRtcErrCameraInterrupt - 采集过程中出现异常，Mac、Winodows电脑相机采集灯熄灭，设备被强制中断。建议重新加入频道或者重启App。 */
    DingRtcErrCameraInterrupt                = 0x01040106,
    /** DingRtcErrScreenShareNoPermission - 用户未给予屏幕共享权限，屏幕共享失败，提示用户授予权限 */
    DingRtcErrScreenShareNoPermission        = 0x01040109,
    /** DingRtcErrDisplayOpenFail - 渲染设备初始化失败，检查Windows端显卡驱动是否需要升级，移动端手机型号或者Android平板盒子等外设是否支持OpenGL。 */
    DingRtcErrDisplayOpenFail                = 0x01040201,
    /** DingRtcErrDisplayInterrupt - 渲染过程中出现异常，系统异常错误导致渲染引擎报错，重新加入频道或者重启App。 */
    DingRtcErrDisplayInterrupt               = 0x01040202,
    /** DingRtcErrIceConnectionConnectFail - 媒体通道建立失败，检查网络连接是否正常。 */
    DingRtcErrIceConnectionConnectFail       = 0x01050201,
    /** DingRtcErrIceConnectionReconnectFail - 媒体通道重连失败，检查网络连接是否正常。 */
    DingRtcErrIceConnectionReconnectFail     = 0x01050202,
    /** DingRtcErrConnectionHeartbeatTimeout - 信令心跳超时，检查网络连接是否正常。 */
    DingRtcErrConnectionHeartbeatTimeout     = 0x0102020C,
    /** DingRtcErrCdnLiveCommunicationFailed - 低延时互动直播信令失败。 */
    DingRtcErrCdnLiveCommunicationFailed     = 0x01070003,
    /** DingRtcErrCdnLiveMeidaSettingFailed - 低延时互动直播媒体设置错误。 */
    DingRtcErrCdnLiveMeidaSettingFailed      = 0x01070004,
    /** DingRtcErrCdnLiveNoneBroadcaster - 低延时互动直播当前房间没有主播推流。 */
    DingRtcErrCdnLiveNoneBroadcaster         = 0x01070005,
    /** DingRtcErrCdnLiveNeedRestart - 低延时互动直播错误，应用层需要重新创建新的SDK实例。 */
    DingRtcErrCdnLiveNeedRestart             = 0x01070006,
    /** DingRtcErrCdnLiveNeedRestartAgainLater - 低延时互动直播错误，应用层需要稍等一定时间（自定义间隔）重新创建新的SDK实例。 */
    DingRtcErrCdnLiveNeedRestartAgainLater   = 0x01070007,
    /** DingRtcErrCdnLiveServiceUnavailable - 低延迟互动直播服务不可用或者服务降级，客户端应该选择其它方式拉超大方会流，即不从cdn sfu拉流。 */
    DingRtcErrCdnLiveServiceUnavailable      = 0x01070008,
    /** DingRtcErrCdnLiveMediaConnectionInterruption - 低延迟互动直播网络中断错误，销毁当前SDK实例并重新创建SDK后重新走订阅流程，已废弃。 */
    DingRtcErrCdnLiveMediaConnectionInterruption = 0x01070009,
    /** DingRtcErrPublishInvaild - 推流无效。 */
    DingRtcErrPublishInvaild                 = 0x01030305,
    /** DingRtcErrPublishNotJoinChannel - 未进入频道推流失败。 */
    DingRtcErrPublishNotJoinChannel          = 0x01010406,
    /** DingRtcErrPublishAudioStreamFailed - 推送音频流失败。 */
    DingRtcErrPublishAudioStreamFailed       = 0x01010450,
    /** DingRtcErrPublishVideoStreamFailed - 推送视频流失败。 */
    DingRtcErrPublishVideoStreamFailed       = 0x01010451,
    /** DingRtcErrPublishScreenShareFailed - 推送屏幕共享失败。 */
    DingRtcErrPublishScreenShareFailed       = 0x01010453,
    /** DingRtcErrPublishScreenShareConfigError - 屏幕共享配置错误。 */
    DingRtcErrPublishScreenShareConfigError  = 0x01010454,
    /** DingRtcErrSubscribeInvaild - 订阅无效。 */
    DingRtcErrSubscribeInvaild               = 0x01030404,
    /** DingRtcErrSubscribeNotJoinChannel - 未进入频道订阅错误。 */
    DingRtcErrSubscribeNotJoinChannel        = 0x01010550,
    /** DingRtcErrSubscribeAudioStreamFailed - 订阅音频流失败。 */
    DingRtcErrSubscribeAudioStreamFailed     = 0x01010551,
    /** DingRtcErrSubscribeVideoStreamFailed - 订阅视频流失败。 */
    DingRtcErrSubscribeVideoStreamFailed     = 0x01010552,
    /** DingRtcErrSubscribeScreenShareFailed - 订阅屏幕共享失败。 */
    DingRtcErrSubscribeScreenShareFailed     = 0x01010554,
    /** DingRtcErrSdkInvalidState - SDK状态错误，建议销毁SDK重新创建实例。 */
    DingRtcErrSdkInvalidState                = 0x01030204,
    /** DingRtcErrInvalidArguments - 参数不匹配。 */
    DingRtcErrInvalidArguments               = 0x01030103,
    /** DingRtcErrSessionRemoved - Session已经被移除，建议销毁实例之后重新创建实例并加入频道，已废弃。 */
    DingRtcErrSessionRemoved                 = 0x02010105,
    /** DingRtcErrAudioBufferFull - buffer队列饱和，用于外部输送裸数据功能，建议降低外部输入频次，设置等待20ms，并且单次不超过40ms数据。 */
    DingRtcErrAudioBufferFull                = 0x01070101,
    /** DingRtcErrVideoBufferFull - buffer队列饱和，用于外部输送裸数据功能，建议降低外部输入频次，设置等待视频帧间隔后再输入。 */
    DingRtcErrVideoBufferFull                = 0x01070102,
    /** DingRtcErrUpdateRoleChannel - 互动模式下设置角色错误。 */
    DingRtcErrUpdateRoleChannel              = 0x01020201,
    /** DingRtcErrInner - SDK内部错误。 */
    DingRtcErrInner                          = -1,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief OnBye类型。
 */
typedef NS_ENUM(NSInteger, DingRtcOnByeType) {
    /** DingRtcOnByeBeKickedOut - 当前user被踢出channel。 */
    DingRtcOnByeBeKickedOut          = 1,
    /** DingRtcOnByeChannelTerminated - channel已结束，需要离开会议。 */
    DingRtcOnByeChannelTerminated    = 2,
    /** DingRtcOnByeUserReplaced - 相同userID在其他设备joinChannel，当前设备被下线。 */
    DingRtcOnByeUserReplaced         = 3,
    /** DingRtcOnByeNeedRelogin - 信令服务失败，自动重连失败，需要用户手动重试。 */
    DingRtcOnByeNeedRelogin          = 4,
};

/**
 * @brief SDK问题反馈类型。
 */
typedef NS_ENUM(NSInteger, DingRtcFeedbackType) {
    /** DingRtcFeedbackVideoDeviceFail视频设备问题。 */
    DingRtcFeedbackVideoDeviceFail   = 101,
    /** 视频无画面。 */
    DingRtcFeedbackVideoNotRender    = 102,
    /** 音频设备问题。 */
    DingRtcFeedbackAudioDeviceFail   = 201,
    /** 音频无声音。 */
    DingRtcFeedbackAudioNotRender    = 202,
    /** 音频回声异常。 */
    DingRtcFeedbackAudioEchoError    = 203,
    /** 会议状态异常：入会异常、离会异常等。 */
    DingRtcFeedbackChannelTypeError  = 301,
    /** SDK其他状态异常。 */
    DingRtcFeedbackSDKTypeError      = 302,
    /** 网络卡顿问题。 */
    DingRtcFeedbackNetworkUnfluent   = 401,
    /** 视频清晰问题。 */
    DingRtcFeedbackVideoBlurring     = 402,
    /** 未知类型。 */
    DingRtcFeedbackUnkonw            = -1,
};

/**
 * @brief 旁路推流LiveStream状态。
 */
typedef NS_ENUM(NSInteger, DingRtcTrascodingLiveTaskStatus) {
    /** 任务开始。 */
    DingRtcTrascodingLiveTaskStatusStart = 0,
    /** 任务更新。 */
    DingRtcTrascodingLiveTaskStatusUpdate = 1,
    /** 任务已停止。 */
    DingRtcTrascodingLiveTaskStatusStop = 2,
};

/**
 * @brief 旁路推流错误码。
 */
typedef NS_ENUM(NSInteger, DingRtcTrascodingLiveStreamErrorCode) {
    DingRtcTrascodingLiveStreamErrorPublishOk            = 0,
    DingRtcTrascodingLiveStreamErrorStreamNotFound       = 0x01100001,
    DingRtcTrascodingLiveStreamErrorStreamAlreadyExist   = 0x01100002,
    DingRtcTrascodingLiveStreamErrorInvalidParam         = 0x01100003,
    DingRtcTrascodingLiveStreamErrorInternalError        = 0x01100004,
    DingRtcTrascodingLiveStreamErrorRtmpServerError      = 0x01100005,
    DingRtcTrascodingLiveStreamErrorRtmpStreamUrlError   = 0x01100006,
    DingRtcTrascodingLiveStreamErrorPublishTimeout       = 0x01100007,
    DingRtcTrascodingLiveStreamErrorNotAuthorized        = 0x01100008,
};

/**
 * @brief 跨频道转推事件。
 */
typedef NS_ENUM(NSInteger, DingRtcChannelRelayEvent) {
    DingRtcChannelRelayStart     = 0,
    DingRtcChannelRelayUpdate    = 1,
    DingRtcChannelRelayStop      = 2,
};

/**
 * @brief 跨频道转推事件。
 */
typedef NS_ENUM(NSInteger, DingRtcChannelRelayState) {
    DingRtcChannelRelayStateInit         = 3,
    DingRtcChannelRelayStateConnecting   = 4,
    DingRtcChannelRelayStateRunning      = 5,
    DingRtcChannelRelayStateFailure      = 6,
};

#if TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark ios
 * @brief 摄像头方向 （仅适用于 iOS 系统）。
 */
typedef NS_ENUM(NSInteger, DingRtcCameraDirection)  {
  /** DingRtcCameraDirectionInvalid - 无效摄像头方向。 */
  DingRtcCameraDirectionInvalid = -1,
  /** DingRtcCameraDirectionBack - 后向摄像头。 */
  DingRtcCameraDirectionBack = 0,
  /** DingRtcCameraDirectionFront - 前向摄像头。 */
  DingRtcCameraDirectionFront = 1,
};
#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 推流状态。
*/
typedef NS_ENUM(NSInteger, DingRtcPublishState) {
    /** DingRtcStatsPublishIdle - 0：SDK初始推流状态。
     */
    DingRtcStatsPublishIdle = 0,
    /** DingRtcStatsNoPublish - 1：未推流状态，可能是因为：已设置停止推送媒体流；推送媒体流失败。
     */
    DingRtcStatsNoPublish   = 1,
    /** DingRtcStatsPublishing - 2：推流链接建立过程中。
     */
    DingRtcStatsPublishing  = 2,
    /** DingRtcStatsPublished - 3：推流成功。
     */
    DingRtcStatsPublished   = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 订阅状态。
*/
typedef NS_ENUM(NSInteger, DingRtcSubscribeState) {
    /** DingRtcStatsSubscribeIdle - 初始状态。
     */
    DingRtcStatsSubscribeIdle = 0,
    /** DingRtcStatsNoSubscribe - 未订阅。
     */
    DingRtcStatsNoSubscribe   = 1,
    /** DingRtcStatsSubscribing - 订阅中。
     */
    DingRtcStatsSubscribing   = 2,
    /** DingRtcStatsSubscribed - 已订阅。
     */
    DingRtcStatsSubscribed    = 3,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 用户离线原因。
*/
typedef NS_ENUM(NSInteger, DingRtcUserOfflineReason) {
    /** DingRtcUserOfflineQuit - 用户主动离开。 */
    DingRtcUserOfflineQuit = 0,
    /** DingRtcUserOfflineDropped - 因过长时间收不到对方数据包，超时掉线。 */
    DingRtcUserOfflineDropped = 1,
    /** DingRtcUserOfflineBecomeAudience - 用户身份从主播切换为观众时触发。 */
    DingRtcUserOfflineBecomeAudience = 2,
};

/**
 * @brief 背景替换背景图缩放模式。
*/
typedef NS_ENUM(NSUInteger, DingRtcBokehScaleModel) {
    /** DingRtcBokehScaleModelCrop - 等比裁剪。 */
    DingRtcBokehScaleModelCrop = 0,
    /** DingRtcBokehScaleModelFill - 填充黑边。 */
    DingRtcBokehScaleModelFill = 1,
};

#if TARGET_OS_MAC && !TARGET_OS_IPHONE
/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 桌面分享类型 （仅适用于 Mac 系统）。
 */
typedef NS_ENUM(NSInteger, DingRtcScreenShareType){
    /** DingRtcScreenShareDesktop - 桌面分享。 */
    DingRtcScreenShareDesktop = 0,
    /** DingRtcScreenShareWindow - 窗口分享。 */
    DingRtcScreenShareWindow  = 1,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 外接设备类型 （仅适用于 Mac 系统）。
 */
typedef NS_ENUM(NSInteger, DingRtcExternalDeviceType) {
    /** DingRtcExternalTypeUnknown - 未知类型。 */
    DingRtcExternalTypeUnknown = -1,
    /** DingRtcExternalTypeAudioCapture - 音频采集设备。 */
    DingRtcExternalTypeAudioCapture = 0,   
    /** DingRtcExternalTypeAudioPlayout - 音频播放设备。 */
    DingRtcExternalTypeAudioPlayout = 1,   
     /** DingRtcExternalTypeVideoCapture - 视频采集设备。 */
    DingRtcExternalTypeVideoCapture = 2,  
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 外接设备状态 （仅适用于 Mac 系统）。
 */
typedef NS_ENUM(NSInteger, DingRtcExternalDeviceState) {
    /** DingRtcExternalDeviceAdd - 新增外接设备。 */
    DingRtcExternalDeviceAdd = 0,   
    /** DingRtcExternalDeviceRemove - 移除外接设备。 */   
    DingRtcExternalDeviceRemove = 1,   
};


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 设备传输类型 （仅适用于 Mac 系统）。
 */
typedef NS_ENUM(NSInteger, DingRtcDeviceTransportType) {
    /** DingRtcDeviceTransportTypeUnknown - 未知类型。 */ 
    DingRtcDeviceTransportTypeUnknown      = 0,
    /** DingRtcDeviceTransportTypeBuiltIn - 内建类型。 */ 
    DingRtcDeviceTransportTypeBuiltIn      = 1,
    /** DingRtcDeviceTransportTypeBluetooth - 蓝牙类型。 */ 
    DingRtcDeviceTransportTypeBluetooth    = 2,
    /** DingRtcDeviceTransportTypeUSB - USB类型。 */ 
    DingRtcDeviceTransportTypeUSB          = 3,
    /** DingRtcDeviceTransportTypeAggregate - 聚合类型。 */ 
    DingRtcDeviceTransportTypeAggregate    = 4,
    /** DingRtcDeviceTransportTypeVirtual - 虚拟类型。 */ 
    DingRtcDeviceTransportTypeVirtual      = 5,
    /** DingRtcDeviceTransportTypePCI - PCI类型。 */ 
    DingRtcDeviceTransportTypePCI          = 6,
    /** DingRtcDeviceTransportTypeFireWire - 火线类型。 */ 
    DingRtcDeviceTransportTypeFireWire     = 7,
    /** DingRtcDeviceTransportTypeBluetoothLE - 蓝牙低能耗类型。 */ 
    DingRtcDeviceTransportTypeBluetoothLE  = 8,
    /** DingRtcDeviceTransportTypeHDMI - HDMI类型。 */ 
    DingRtcDeviceTransportTypeHDMI         = 9,
    /** DingRtcDeviceTransportTypeDisplayPort - DP类型。 */ 
    DingRtcDeviceTransportTypeDisplayPort  = 10,
    /** DingRtcDeviceTransportTypeAirPlay - AirPlay类型。 */ 
    DingRtcDeviceTransportTypeAirPlay      = 11,
    /** DingRtcDeviceTransportTypeAVB - AVB 类型。 */ 
    DingRtcDeviceTransportTypeAVB          = 12,
    /** DingRtcDeviceTransportTypeThunderbolt - 雷电类型。 */ 
    DingRtcDeviceTransportTypeThunderbolt  = 13
};

#endif

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 质量报告数据。
*/
__attribute__((visibility("default"))) @interface DingRtcQualityReport : NSObject

// cpu
/** CPU物理核数。 */
@property (nonatomic, assign) int cpu_physical_cores;
/** CPU逻辑核数。 */
@property (nonatomic, assign) int cpu_logic_cores;
/** CPU频率（单位赫兹）。 */
@property (nonatomic, assign) int cpu_frequency_mhz;
/** CPU本应用使用比例（百分比）。 */
@property (nonatomic, assign) int cpu_used_by_app;
/** CPU其他应用使用比例（百分比）。 */
@property (nonatomic, assign) int cpu_used_by_other_apps;

// memory
/** 总使用内存（单位MB）。 */
@property (nonatomic, assign) int memory_total_mb;
/** 本应用使用内存（单位MB）。 */
@property (nonatomic, assign) int memory_used_by_app;
/** 其他应用使用内存（单位MB）。 */
@property (nonatomic, assign) int memory_used_by_other_apps;

// network
/** 丢包率。 */
@property (nonatomic, assign) int loss_rate; // percentage
/** 延时。 */
@property (nonatomic, assign) int rtt_ms;
/** 接收码率。 */
@property (nonatomic, assign) int recv_bitrate;
/** 发送码率。 */
@property (nonatomic, assign) int send_bitrate;
/** 信令级别。 */
@property (nonatomic, assign) int signal_level;
/** 网络类型。 */
@property (nonatomic, copy) NSString * _Nonnull network_type;

// audio up
/** 音频发送码率。 */
@property (nonatomic, assign) int audio_send_bitrate;
/** 音频发送音量。 */
@property (nonatomic, assign) int audio_send_volume; // in db

// audio down
/** 音频接收码率。 */
@property (nonatomic, assign) int audio_recv_bitrate;
/** 音频接收音量。 */
@property (nonatomic, assign) int audio_recv_volume; // in db

// video up
/** 视频发送码率。 */
@property (nonatomic, assign) int camera_send_bitrate;
/** 视频发送的采集帧率。 */
@property (nonatomic, assign) int camera_send_capture_fps;
@property (nonatomic, assign) int camera_send_drop_fps;
/** 视频发送帧率。 */
@property (nonatomic, assign) int camera_send_fps;
@property (nonatomic, assign) int camera_send_hw;
/** 视频发送宽度。 */
@property (nonatomic, assign) int camera_send_width;
/** 视频发送高度。 */
@property (nonatomic, assign) int camera_send_height;

/** 屏幕共享发送码率。 */
@property (nonatomic, assign) int screenshare_send_bitrate;
/** 屏幕共享采集帧率。 */
@property (nonatomic, assign) int screenshare_send_capture_fps;
@property (nonatomic, assign) int screenshare_send_drop_fps;
/** 屏幕共享发送帧率。 */
@property (nonatomic, assign) int screenshare_send_fps;
@property (nonatomic, assign) int screenshare_send_hw;
/** 屏幕共享宽度。 */
@property (nonatomic, assign) int screenshare_send_width;
/** 屏幕共享高度。 */
@property (nonatomic, assign) int screenshare_send_height;

// video down
/** 视频接收码率。 */
@property (nonatomic, assign) int camera_recv_bitrate; // camera stream, recv bitrate
/** 视频接收帧率。 */
@property (nonatomic, assign) int camera_recv_fps;
/** 视频渲染器帧率。 */
@property (nonatomic, assign) int camera_recv_render_fps;
@property (nonatomic, assign) int camera_recv_hw;
/** 视频接收宽度。 */
@property (nonatomic, assign) int camera_recv_width;
/** 视频接收高度。 */
@property (nonatomic, assign) int camera_recv_height;

/** 屏幕共享接收码率。 */
@property (nonatomic, assign) int screenshare_recv_bitrate; // screen stream, recv bitrate
/** 屏幕共享接收帧率。 */
@property (nonatomic, assign) int screenshare_recv_fps;
/** 屏幕共享渲染帧率。 */
@property (nonatomic, assign) int screenshare_recv_render_fps;
@property (nonatomic, assign) int screenshare_recv_hw;
/** 屏幕共享接收宽度。 */
@property (nonatomic, assign) int screenshare_recv_width;
/** 屏幕共享接收高度。 */
@property (nonatomic, assign) int screenshare_recv_height;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 基础统计信息。
 */
__attribute__((visibility("default"))) @interface DingRtcStats : NSObject
/** 可用带宽（kb）。 */
@property (nonatomic, assign) long long available_sent_kbitrate;
/** 总发送码率（kb）。 */
@property (nonatomic, assign) long long sent_kbitrate;
/** 总接收码率（kb）。 */
@property (nonatomic, assign) long long recv_kbitrate;
/** 总发送数据量（bytes）。 */
@property (nonatomic, assign) long long sent_bytes;
/** 总接收数据量（bytes）。 */
@property (nonatomic, assign) long long recv_bytes;
/** 视频接收码率（kb）。 */
@property (nonatomic, assign) long long video_recv_kbitrate;
/** 视频发送码率（kb）。 */
@property (nonatomic, assign) long long video_sent_kbitrate;
/** 音频接收码率（kb）。 */
@property (nonatomic, assign) long long audio_recv_kbitrate;
/** 音频发送码率（kb）。 */
@property (nonatomic, assign) long long audio_sent_kbitrate;
/** 通话时长(s)。 */
@property (nonatomic, assign) long long call_duration;
/** 进程CPU使用量 。*/
@property (nonatomic, assign) unsigned int cpu_usage;
/** 系统CPU使用量。 */
@property (nonatomic, assign) unsigned int systemCpuUsage;
/** 客户端到服务器的丢包率。 */
@property (nonatomic, assign) long long sent_loss_rate;
/** 客户端到服务器的丢包数。 */
@property (nonatomic, assign) long long sent_loss_pkts;
/** 客户端到服务器的总包数。 */
@property (nonatomic, assign) long long sent_expected_pkts;
/** 客户端到服务器的下行丢包率。 */
@property (nonatomic, assign) long long recv_loss_rate;
/** 客户端到服务器的下行丢包数。 */
@property (nonatomic, assign) long long recv_loss_pkts;
/** 客户端到服务器的下行总包数。 */
@property (nonatomic, assign) long long recv_expected_pkts;
/** 客户端到服务器的延迟（ms）。 */
@property (nonatomic, assign) long long lastmile_delay;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 本地视频统计信息。
 */
__attribute__((visibility("default"))) @interface DingRtcLocalVideoStats : NSObject

/** 视频流track类型。 */
@property (nonatomic, assign) DingRtcVideoTrack track;
/** 目标编码码率。 */
@property (nonatomic, assign) unsigned int targetEncodeBitrate;
/** 实际编码码率。 */
@property (nonatomic, assign) unsigned int actualEncodeBitrate;
/** 发送码率。 */
@property (nonatomic, assign) unsigned int sentBitrate;
/** 采集帧率。 */
@property (nonatomic, assign) unsigned int captureFps;
/** 发送帧率。 */
@property (nonatomic, assign) unsigned int sentFps;
/** 编码器输出帧率。 */
@property (nonatomic, assign) unsigned int encodeFps;
/** 渲染帧率。 */
@property (nonatomic, assign) unsigned int renderFps;
/** 每秒平均QP。 */
@property (nonatomic, assign) unsigned int avgQpPerSec;
/** 编码帧宽。 */
@property (nonatomic, assign) unsigned int encoderFrameWidth;
/** 编码帧高。 */
@property (nonatomic, assign) unsigned int encoderFrameHeight;
/** 采集帧宽。 */
@property (nonatomic, assign) unsigned int captureFrameWidth;
/** 采集帧高。 */
@property (nonatomic, assign) unsigned int captureFrameHeight;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 远端视频统计信息。
 */
__attribute__((visibility("default"))) @interface DingRtcRemoteVideoStats : NSObject
/** 远端用户userID。 */
@property (nonatomic, copy) NSString * _Nonnull userId;
/** 视频流track类型。 */
@property (nonatomic, assign) DingRtcVideoTrack track;
/** 视频帧的宽。 */
@property (nonatomic, assign) unsigned int width;
/** 视频帧的高。 */
@property (nonatomic, assign) unsigned int height;
/** 接收码率。 */
@property (nonatomic, assign) unsigned int recvBitrate;
/** 解码帧率。 */
@property (nonatomic, assign) unsigned int decoderOutputFrameRate;
/** 渲染帧率。 */
@property (nonatomic, assign) unsigned int rendererOutputFrameRate;
/** 丢包率。 */
@property (nonatomic, assign) unsigned int packetLossRate;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 本地音频统计信息。
 */
__attribute__((visibility("default"))) @interface DingRtcLocalAudioStats : NSObject

/** 音频流track类型。 */
@property (nonatomic, assign) DingRtcAudioTrack track;
/** 声道数。 */
@property (nonatomic, assign) unsigned int numChannel;
/** 采样率 */
@property (nonatomic, assign) unsigned int sentSamplerate;
/** 发送端音频的瞬时码率。 */
@property (nonatomic, assign) unsigned int sentBitrate;
/** 输入音量。 */
@property (nonatomic, assign) unsigned int inputLevel;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 远端音频统计信息。
 */
__attribute__((visibility("default"))) @interface DingRtcRemoteAudioStats : NSObject

/** 远端用户userID。 */
@property (nonatomic, copy) NSString * _Nonnull userId;
/** 回调周期内的音频丢帧率。 */
@property (nonatomic, assign) unsigned int packetLossRate;
/** 接收流的瞬时码率。 */
@property (nonatomic, assign) unsigned int recvBitrate;
/** 远端用户加入频道后音频卡顿的累计时长 （ms）。 */
@property (nonatomic, assign) unsigned int totalFrozenTime;
/** 远端用户加入频道后音频非静音样本拉伸率。 */
@property (nonatomic, assign) unsigned int speechExpandRate;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 发言人音量信息。
 */
__attribute__((visibility("default"))) @interface DingRtcUserVolumeInfo : NSObject

/** 发言人的用户ID， userId为"0"表示本地用户。 */
@property (nonatomic, copy) NSString * _Nonnull userId;
/** 发言状态，YES：正在说话，NO：没有说话。 */
@property (nonatomic, assign) BOOL speechState;
/** 音量值，取值范围[0,100]。 */
@property (nonatomic, assign) int volume;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.3.0
 * @brief 音频回调数据源参数。
 */
__attribute__((visibility("default"))) @interface DingRtcAudioObserveConfig : NSObject

/** 回调数据的采样率。 */
@property (nonatomic, assign) DingRtcAudioSampleRate sampleRate;
/** 回调数据的通道数。 */
@property (nonatomic, assign) DingRtcAudioNumChannel channel;
/** 回调数据的权限。 */
@property (nonatomic, assign) DingRtcAudioFramePermission permission;

@end


/**
 * @ingroup OC_DingRtcDef
 * @since 3.4
 * @brief 音乐伴奏混音状态信息。
 */
__attribute__((visibility("default"))) @interface DingRtcAudioMixingStatusConfig : NSObject

/** 伴奏混音状态。 */
@property (nonatomic, assign) DingRtcAudioMixingStatus status;
/** 伴奏混音错误信息。 */
@property (nonatomic, assign) DingRtcAudioMixingErrorCode errorCode;
/** 伴奏文件名称。 */
@property (nonatomic, copy) NSString * _Nonnull fileName;
/** 伴奏文件id。 */
@property (nonatomic, assign) NSInteger uniqueId;
/** 伴奏时长。 */
@property (nonatomic, assign) long durationMs;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.4
 * @brief 音乐伴奏混音状态信息。
 */
__attribute__((visibility("default"))) @interface DingRtcAudioMixingConfig : NSObject

/** 是否允许伴奏推流到远端，YES：允许推流，NO：不允许推流（默认）。 */
@property (nonatomic, assign) BOOL enablePublish;
/** 是否允许伴奏本地播放，YES：允许播放（默认），NO：不允许播放。 */
@property (nonatomic, assign) BOOL enablePlayout;
/** 伴奏推流到远端音量，取值范围[0, 100]，默认100。 */
@property (nonatomic, assign) int publishVolume;
/** 伴奏本地播放音量，取值范围[0, 100]，默认100。 */
@property (nonatomic, assign) int playoutVolume;
/** 循环次数， 大于0有效，默认1 */
@property (nonatomic, assign) int cycles;
/** 开始位置，单位ms，默认0。 */
@property (nonatomic, assign) long startPosMs;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 入会鉴权信息。
 */
__attribute__((visibility("default"))) @interface DingRtcAuthInfo : NSObject <NSCopying>

/** 频道ID。 */
@property (nonatomic, copy) NSString * _Nonnull channelId;
/** 用户ID。 */
@property (nonatomic, copy) NSString * _Nonnull userId;
/** 应用ID。 */
@property (nonatomic, copy) NSString * _Nonnull appId;
/** 令牌。 */
@property (nonatomic, copy) NSString * _Nonnull token;
/** GSLB地址。 */
@property (nonatomic, copy) NSString * _Nullable gslbServer;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 显示视图配置。
*/
__attribute__((visibility("default"))) @interface DingRtcVideoCanvas : NSObject
/** 显示视图。 */
@property (nonatomic, strong) DingRtcOSView * _Nonnull view;
/** 渲染模式，默认值为{@link DingRtcRenderMode::DingRtcRenderModeAuto}。 */
@property (nonatomic) DingRtcRenderMode renderMode;
/** 镜像模式，默认值为{@link DingRtcRenderMirrorMode::DingRtcRenderMirrorModeOnlyFrontCameraPreviewEnabled}。 */
@property (nonatomic) DingRtcRenderMirrorMode mirrorMode;
/** 背景颜色，格式为RGB的Hex。 */
@property (nonatomic) int backgroundColor;
/** 旋转角度，默认值为 {@link DingRtcRotationMode::DingRtcRotationMode_0}。 */
@property (nonatomic, assign) DingRtcRotationMode rotationMode;
/** 是否移除。*/
@property (nonatomic, assign) BOOL toBeRemoved;

@end

/**
 * @ingroup OC_DingRtcDef
 *
 * @since 3.0
 * @brief 水印配置。
*/
__attribute__((visibility("default"))) @interface DingRtcWatermarkConfig : NSObject
/** 预览水印是否可见。  */
@property (nonatomic, assign) BOOL visibleInPreview;
/** 竖屏水印坐标信息。  */
@property (nonatomic, assign) CGRect positionInPortraitMode;
/** 横屏水印坐标信息。  */
@property (nonatomic, assign) CGRect positionInLandscapeMode;
/** 水印透明度。  */
@property (nonatomic, assign) float alpha;
/** 归一化 。 */
@property (nonatomic, assign) BOOL normalized;

@end


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 音频数据采样。
 */
__attribute__((visibility("default"))) @interface DingRtcAudioDataSample : NSObject

/** 音频数据。 */
@property (nonatomic, assign) void* _Nonnull data;
/** 采样点数（每个声道）。*/
@property (nonatomic, assign) int numOfSamples;
/** 每个采样点字节数，通常是16bit（即2字节）。*/
@property (nonatomic, assign) int bytesPerSample;
/** 声道数。 */
@property (nonatomic, assign) int numOfChannels;
/** 采样率。 */
@property (nonatomic, assign) int sampleRate;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 摄像头采集偏好。
 */
__attribute__((visibility("default"))) @interface DingRtcCameraCapturerConfiguration : NSObject

#if TARGET_OS_IPHONE
/** 摄像头方向 （仅适用于 iOS 系统） 。*/
@property (nonatomic, assign) DingRtcCameraDirection cameraDirection;
#endif

@end

typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingDisplayType) {
    NOT_DISPLAY = 0,
    ALWAYS = 1,
    WHEN_NO_VIDEO = 2
};

/**
 * @brief 旁路直播图片。
 */
__attribute__((visibility("default"))) @interface DingRtcLiveTranscodingImage : NSObject
@property (nonatomic, copy) NSString *_Nullable url;
@property (nonatomic, assign) float alpha;
@property (nonatomic, assign) DingRtcLiveTranscodingDisplayType display;
@property (nonatomic, assign) int x;
@property (nonatomic, assign) int y;
@property (nonatomic, assign) int width;
@property (nonatomic, assign) int height;
@property (nonatomic, assign) int zOrder;
@end

typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingFontType) {
  NOTO_SERIF_CJKSC_REGULAR = 0,
  ALIBABA_PUHUITI_REGULAR = 1,
  ALIBABA_PUHUITI_BOLD = 2,
  ALIBABA_PUHUITI_Heavy = 3,
  ALIBABA_PUHUITI_LIGHT = 4,
  ALIBABA_PUHUITI_MEDIUM = 5
};

/**
 * @brief 旁路直播文字格式。
 */
__attribute__((visibility("default"))) @interface DingRtcLiveTranscodingText : NSObject
@property (nonatomic, copy) NSString *_Nullable text;
@property (nonatomic, assign) int x;
@property (nonatomic, assign) int y;
@property (nonatomic, assign) int zOrder;
@property (nonatomic, assign) DingRtcLiveTranscodingFontType fontType;
@property (nonatomic, assign) int fontSize;
@property (nonatomic, assign) int fontColor;//0xRRGGBB
@end

/**
 * @brief 旁路直播输入类型。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingSegmentType) {
    /** 无人像分割。 */
    DingRtcLiveTranscodingNoBody = 0,
    /** 人像分割。 */
    DingRtcLiveTranscodingBody = 1
};

/**
 * @brief 旁路直播输入类型。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingSourceType) {
    /** 相机流。 */
    DingRtcLiveTranscodingCamera = 0,
    /** 屏幕流。 */
    DingRtcLiveTranscodingShareScreen = 1
};

/**
 * @brief 旁路直播用户自定义窗格信息。
*/
__attribute__((visibility("default"))) @interface DingRtcLiveTranscodingPane : NSObject
@property (nonatomic, assign) int x;
@property (nonatomic, assign) int y;
@property (nonatomic, assign) int width;
@property (nonatomic, assign) int height;
@property (nonatomic, assign) int zOrder;
@property (nonatomic, copy) NSString *_Nonnull userId;
@property (nonatomic, assign) DingRtcLiveTranscodingSourceType sourceType;
@property (nonatomic, assign) DingRtcLiveTranscodingSegmentType segmentType;
@property (nonatomic, copy) NSArray<DingRtcLiveTranscodingImage *> *_Nullable images;
@property (nonatomic, copy) NSArray<DingRtcLiveTranscodingText *> *_Nullable texts;

@end

/** 旁路直播。 **/
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingState) {
    DingRtcLiveTranscodingState_IDLE = 0,
    DingRtcLiveTranscodingState_CONNNECT = 1,
    DingRtcLiveTranscodingState_RUNNING = 2,
    DingRtcLiveTranscodingState_RECOVERING = 3,
    DingRtcLiveTranscodingState_FAILURE = 4,
    DingRtcLiveTranscodingState_END = 5
};

/**
 * @brief 旁路模式。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingMixMode) {
    /** 单路模式。 */
    DingRtcLiveTranscodingSINGLE = 0,
    /** 混流模式。 */
    DingRtcLiveTranscodingMIX = 1
};

/**
 * @brief 旁路直播转推流模式。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingStreamType) {
    /** 原始流。 */
    DingRtcLiveTranscodingOrigin = 0,
    /** 音频。 */
    DingRtcLiveTranscodingAudio = 1,
    /** 视频。 */
    DingRtcLiveTranscodingVideo = 2
};

/**
 * @brief 旁路直播媒体处理模式。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingMediaProcessMode) {
    /** 通用模式。 */
    DingRtcLiveTranscodingNormal = 0,
    /** 虚拟背景模式。 */
    DingRtcLiveTranscodingVirtualBackground = 1
};

/**
 * @brief 旁路直播裁剪模式。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingCropMode) {
    /** 剪裁。 */
    DingRtcLiveTranscodingCrop = 1,
    /** 填充。 */
    DingRtcLiveTranscodingFill = 2
};

/**
 * @brief 旁路直播时钟格式。
 */
__attribute__((visibility("default"))) @interface DingRtcLiveTranscodingClockWidget : NSObject
@property (nonatomic, assign) int x;
@property (nonatomic, assign) int y;
@property (nonatomic, assign) DingRtcLiveTranscodingFontType fontType;
@property (nonatomic, assign) int fontSize;
@property (nonatomic, assign) int fontColor;  // 0xRRGGBB
@property (nonatomic, assign) int zOrder;
@end

/**
 * @brief 旁路直播音频采样。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingAudioSampleRate) {
    DingRtcLiveTranscoding_HZ_48000 = 0,
    DingRtcLiveTranscoding_HZ_44100 = 1,
    DingRtcLiveTranscoding_HZ_32000 = 2,
    DingRtcLiveTranscoding_HZ_16000 = 3,
    DingRtcLiveTranscoding_HZ_8000 = 4
};

/**
 * @brief 旁路直播自定义编码参数。
 */
__attribute__((visibility("default"))) @interface DingRtcLiveTranscodingEncodeParam : NSObject
@property (nonatomic, assign) int videoWidth; //(0-1920]
@property (nonatomic, assign) int videoHeight; //(0-1920]
@property (nonatomic, assign) int videoFramerate; //[1,60]
@property (nonatomic, assign) int videoBitrate; //[1kbps,10000kbps]
@property (nonatomic, assign) int videoGop; //[1,60]
@property (nonatomic, assign) DingRtcLiveTranscodingAudioSampleRate audioSamplerate;
@property (nonatomic, assign) int audioBitrate; //[8kbps,500kbps]
@property (nonatomic, assign) int audioChannels; //[1,2]
@end

/**
 * @brief 旁路直播计费规格。
 */
typedef NS_ENUM(NSUInteger, DingRtcLiveTranscodingTaskProfile) {
    DingRtcLiveTranscoding_Profile_1IN_1080P = 0,
    DingRtcLiveTranscoding_Profile_1IN_720P = 1,
    DingRtcLiveTranscoding_Profile_1IN_360P = 2,
    DingRtcLiveTranscoding_Profile_2IN_1080P = 3,
    DingRtcLiveTranscoding_Profile_2IN_720P = 4,
    DingRtcLiveTranscoding_Profile_2IN_360P = 5,
    DingRtcLiveTranscoding_Profile_4IN_1080P = 6,
    DingRtcLiveTranscoding_Profile_4IN_720P = 7,
    DingRtcLiveTranscoding_Profile_4IN_360P = 8,
    DingRtcLiveTranscoding_Profile_9IN_1080P = 9,
    DingRtcLiveTranscoding_Profile_9IN_720P = 10,
    DingRtcLiveTranscoding_Profile_9IN_360P = 11,
    DingRtcLiveTranscoding_Profile_12IN_1080P = 12,
    DingRtcLiveTranscoding_Profile_12IN_720P = 13,
    DingRtcLiveTranscoding_Profile_12IN_360P = 14,
    DingRtcLiveTranscoding_Profile_16IN_1080P = 15,
    DingRtcLiveTranscoding_Profile_16IN_720P = 16,
    DingRtcLiveTranscoding_Profile_16IN_360P = 17,
    DingRtcLiveTranscoding_Profile_Mixed = 9999 // only audio
};

/**
 * @brief 旁路模式单路参数。
 */
__attribute__((visibility("default"))) @interface DingRtcLiveTranscodingSingleParam : NSObject
/** 用户id。 */
@property (nonatomic, copy) NSString *_Nullable userId;
/** 推流模式。 */
@property (nonatomic, assign) DingRtcLiveTranscodingStreamType streamType;
/** 旁路直播输入类型。 */
@property (nonatomic, assign) DingRtcLiveTranscodingSourceType sourceType;
@end

/**
 * @brief 旁路模式混流参数。
 */
__attribute__((visibility("default"))) @interface DingRtcLiveTranscodingMixParam : NSObject
/** 旁路直播计费规格。 */
@property (nonatomic, assign) DingRtcLiveTranscodingTaskProfile taskProfile;
/** 旁路直播自定义编码参数。 */
@property (nonatomic, strong) DingRtcLiveTranscodingEncodeParam *_Nullable encodeParam;
/** 旁路直播用户列表。 */
@property (nonatomic, strong) NSArray<DingRtcLiveTranscodingPane *> *_Nullable users;
/** 背景色。 */
@property (nonatomic, assign) int backgroundColor; //0xRRGGBB
/** 背景图片列表。 */
@property (nonatomic, strong) NSArray<DingRtcLiveTranscodingImage *> *_Nullable backgrounds;
/** 水印列表。 */
@property (nonatomic, strong) NSArray<DingRtcLiveTranscodingImage *> *_Nullable watermarks;
/** 时钟组件列表。 */
@property (nonatomic, strong) NSArray<DingRtcLiveTranscodingClockWidget *> *_Nullable clockWidgets;
/** 剪裁模式。 */
@property (nonatomic, assign) DingRtcLiveTranscodingCropMode cropMode;
/** 媒体处理模式。 */
@property (nonatomic, assign) DingRtcLiveTranscodingMediaProcessMode mediaProcessMode;
@end

/**
 * @brief 旁路模式参数。
 */
__attribute__((visibility("default"))) @interface DingRtcLiveTranscodingParam : NSObject
/** 旁路模式。 */
@property (nonatomic, assign) DingRtcLiveTranscodingMixMode mixMode;
/** 路模式单路参数。 */
@property (nonatomic, strong) DingRtcLiveTranscodingSingleParam *_Nullable singleParam;
/** 旁路模式混流参数。 */
@property (nonatomic, strong) DingRtcLiveTranscodingMixParam *_Nullable mixParam;
@end

/**
 * @ingroup OC_DingRtcDef
 *
 * @since 3.0
 * @brief 预定义视频分辨率。
 */
static const CGSize DingRtcVideoDimension120x120 = {120, 120};
static const CGSize DingRtcVideoDimension160x120 = {160, 120};
static const CGSize DingRtcVideoDimension180x180 = {180, 180};
static const CGSize DingRtcVideoDimension240x180 = {240, 180};
static const CGSize DingRtcVideoDimension320x180 = {320, 180};
static const CGSize DingRtcVideoDimension240x240 = {240, 240};
static const CGSize DingRtcVideoDimension320x240 = {320, 240};
static const CGSize DingRtcVideoDimension424x240 = {424, 240};
static const CGSize DingRtcVideoDimension360x360 = {360, 360};
static const CGSize DingRtcVideoDimension480x360 = {480, 360};
static const CGSize DingRtcVideoDimension640x360 = {640, 360};
static const CGSize DingRtcVideoDimension480x480 = {480, 480};
static const CGSize DingRtcVideoDimension640x480 = {640, 480};
static const CGSize DingRtcVideoDimension840x480 = {840, 480};
static const CGSize DingRtcVideoDimension960x540 = {960, 540};
static const CGSize DingRtcVideoDimension960x720 = {960, 720};
static const CGSize DingRtcVideoDimension1280x720 = {1280, 720};
static const CGSize DingRtcVideoDimension1920x1080 = {1920, 1080};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 相机流视频编码属性设置。
 */
__attribute__((visibility("default"))) @interface DingRtcVideoEncoderConfiguration : NSObject

/** 视频分辨率，默认值640x480，可自行设置分辨率，也可以选择上面预定义的分辨率。
*/
@property (nonatomic, assign) CGSize dimensions;

/** 视频帧率，默认值15, 最大值30。
*/
@property (nonatomic, assign) NSInteger frameRate;

/** 视频输出方向，默认DingRtcVideoEncoderOrientationModeAdaptive，详见 {@link DingRtcVideoEncoderOrientationMode}。
*/
@property (nonatomic, assign) DingRtcVideoEncoderOrientationMode orientationMode;

/** 推流旋转，默认DingRtcRotationMode_0，详见 {@link DingRtcRotationMode}。
*/
@property (nonatomic, assign) DingRtcRotationMode rotationMode;

/** 推流镜像，默认DingRtcVideoEncoderMirrorMode_Disabled，详见 {@link DingRtcVideoEncoderMirrorMode}。
*/
@property (nonatomic, assign) DingRtcVideoEncoderMirrorMode mirrorMode;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 屏幕流编码属性设置。
 */
__attribute__((visibility("default"))) @interface DingRtcScreenShareEncoderConfiguration : NSObject

/** 视频分辨率，默认值0x0，最大值3840x2160。
 @note
 默认值表示推流分辨率等于屏幕采集的分辨率。
*/
@property (nonatomic, assign) CGSize dimensions;

/** 视频帧率，默认值5, 最大值30。
 */
@property (nonatomic, assign) NSInteger frameRate;

/** 推流旋转，默认DingRtcRotationMode_0，详见 {@link DingRtcRotationMode}。
*/
@property (nonatomic, assign) DingRtcRotationMode rotationMode;
@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频状态。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoState){
    /** DingRtcVideoClose - 视频关闭。 */
    DingRtcVideoClose   = 0,
    /** DingRtcVideoOpen - 视频打开。 */
    DingRtcVideoOpen    = 1,
};

/**
 * @brief 视频状态变化原因
 */
typedef NS_ENUM(NSInteger, DingRtcVideoReason){
    /** 由客户端触发。 */
    DingRtcVideoChangeByClient    = 0,
    /** 由服务器触发。 */
    DingRtcVideoChangeByServer    = 1,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief Qos反馈外置编码器请求帧类型
 */
typedef NS_ENUM(NSInteger, DingRtcVideoEncodedFrameType) {
    /** DingRtcVideoEncodedFrameNULL - 默认：无。 */
    DingRtcVideoEncodedFrameNULL,
    /** DingRtcVideoEncodedFrameIDR - IDR帧。 */
    DingRtcVideoEncodedFrameIDR,
    /** DingRtcVideoEncodedFrameLTR - LTR帧。 */
    DingRtcVideoEncodedFrameLTR,
    /** DingRtcVideoEncodedFrameB - B帧。 */
    DingRtcVideoEncodedFrameB,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief Qos反馈外置编码器参数结构体。
 */
__attribute__((visibility("default"))) @interface DingRtcVideoExternalEncoderParameter : NSObject
/** 视频帧的宽。
 */
@property (nonatomic, assign) NSInteger width;
/** 视频帧的高。
 */
@property (nonatomic, assign) NSInteger height;
/** 编码帧率。
 */
@property (nonatomic, assign) NSInteger frame_rate;
/** 码率。
 */
@property (nonatomic, assign) NSInteger bitrate_bps;
@end

/**
 * @brief 视频美颜选项。
 */
__attribute__((visibility("default"))) @interface DingRtcBeautyFaceOptions : NSObject

/** 资源文件路径。 */
@property (nonatomic, strong) NSString * _Nonnull resourcePath;
/** 是否开启美肤。 */
@property (nonatomic, assign) BOOL enableSkinBuffing;
/** 磨皮系数，取值范围[0-1]。 */
@property (nonatomic, assign) float skinBuffingFactor;
/** 锐化系数，取值范围[0-1]。 */
@property (nonatomic, assign) float skinSharpenFactor;
/** 是否开启美白。 */
@property (nonatomic, assign) BOOL enableSkinWhitening;
/** 美白系数，取值范围[0-1]。 */
@property (nonatomic, assign) float skinWhitingFactor;
@end

/**
 * @brief 视频虚拟背景模式
 */
typedef NS_ENUM(NSInteger, DingRtcVirtualBackgroundMode) {
    /** 背景虚化。 */
    DingRtcVirtualBackgroundBlur = 0,
    /** 背景替换。 */
    DingRtcVirtualBackgroundReplace = 1,
};

/**
 * @brief 视频虚拟背景选项
 */
__attribute__((visibility("default"))) @interface DingRtcVirtualBackgroundOptions : NSObject

/** 虚拟背景模式。 */
@property (nonatomic, assign) DingRtcVirtualBackgroundMode mode;
/** 背景文件路径。 */
@property (nonatomic, strong) NSString * _Nonnull bgFilePath;
@end

/**
 * @brief 视频暗光增强选项。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoEnhanceMode) {
    /** 视频暗光增强关。 */
    DingRtcVideoEnhanceOff,
    /** 视频暗光增强自动模式。 */
    DingRtcVideoEnhanceAuto,
};

/**
 * @brief 视频暗光增强选项。
 */
__attribute__((visibility("default"))) @interface DingRtcVideoEnhanceOptions : NSObject
/** 视频暗光增强模式。 */
@property (nonatomic, assign) DingRtcVideoEnhanceMode mode;
@end

/**
 * @brief 视频去噪模式。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoDenoiseMode) {
    /** 视频去噪关。 */
    DingRtcVideoDenoiseOff,
    /** 视频去噪自动模式。 */
    DingRtcVideoDenoiseAuto,
};

/**
 * @brief 音频去噪模式。
 */
typedef NS_ENUM(NSInteger, DingRtcAudioDenoiseMode) {
    /** 音频去噪关。 */
    DingRtcAudioDenoiseOff = 0,
    /** 音频去噪采用传统降噪。 */
    DingRtcAudioDenoiseDsp = 1,
    /** 音频去噪采用智能降噪。 */
    DingRtcAudioDenoiseEnhance = 2,
};

/**
 * @brief 视频去噪选项。
 */
__attribute__((visibility("default"))) @interface DingRtcVideoDenoiseOptions : NSObject
/** 视频去噪模式。 */
@property (nonatomic, assign) DingRtcVideoDenoiseMode mode;
@end

/**
 * @brief 音频去噪选项。
 */
__attribute__((visibility("default"))) @interface DingRtcAudioDenoiseOptions : NSObject
/** 音频去噪模式。 */
@property (nonatomic, assign) DingRtcAudioDenoiseMode mode;
@end


/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频帧类型。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoFrameType) {
  /** DingRtcVideoFrameRaw - 裸数据类型。 */
  DingRtcVideoFrameRaw = 0,
  /** DingRtcVideoFramePixelBuffer - CVPixelBufferRef 类型。 */
  DingRtcVideoFramePixelBuffer = 1,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频数据格式。
 */
typedef NS_ENUM(NSInteger, DingRtcVideoPixelFormat) {
  /** DingRtcVideoI420 - I420。 */
  DingRtcVideoI420,
  /** DingRtcVideoNV12 - NV12。 */
  DingRtcVideoNV12,
  /** DingRtcVideoBGRA - BGRA。 */
  DingRtcVideoBGRA,
};

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @brief 视频帧数据结构。
 */
__attribute__((visibility("default"))) @interface DingRtcVideoFrame : NSObject

@property (nonatomic, assign) DingRtcVideoFrameType frameType;
@property (nonatomic, assign) DingRtcVideoPixelFormat pixelFormat;
/** 时间戳。 */
@property (nonatomic, assign) long long timestamp;
/** 视频帧的宽。 */
@property (nonatomic, assign) int width;
/** 视频帧的高。 */
@property (nonatomic, assign) int height;
/** 视频帧的旋转角度。 */
@property (nonatomic, assign) int rotation;
/** 视频帧的数量。 */
@property (nonatomic, assign) int count;
/** 视频帧数据。 */
@property (nonatomic, assign) void* _Nonnull data;
@property (nonatomic, strong) NSArray<NSNumber *> * _Nullable offset;
/** 视频帧行间距。 */
@property (nonatomic, strong) NSArray<NSNumber *> * _Nullable stride;
/** pixelBuffer数据。 */
@property (nonatomic, assign) CVPixelBufferRef _Nullable pixelBuffer;

@end

__attribute__((visibility("default"))) @interface DingRtcRecordOptions : NSObject
@property (nonatomic, assign) unsigned int width;
@property (nonatomic, assign) unsigned int height;
@property (nonatomic, assign) DingRtcRecordType recordMode;
@property (nonatomic, assign) int videoFps;
@property (nonatomic, assign) BOOL useHardwareAccelerate;
@end

/**
 * @brief 录制视频背景色 （仅适用于 Mac 系统）。
 */
typedef struct  {
    /** 红色分量。 */
    uint8_t r;
    /** 绿色分量。 */
    uint8_t g;
    /** 蓝色分量。 */
    uint8_t b;
} DingRtcRecordVideoBgColor;

/**
 * @brief 录制视频布局 （仅适用于 Mac 系统）。
 */
typedef NS_ENUM(NSUInteger, DingRtcRecordVideoLayoutMode) {
    /** 网格布局。 */
    DingRtcRecordVideoLayoutMode_GRID = 0,
    /** 演讲者布局。 */
    DingRtcRecordVideoLayoutMode_SPEAKER = 1,
    /** 只有一个画面布局。 */
    DingRtcRecordVideoLayoutMode_UNIQUE = 2,
    /** 自定义布局。 */
    DingRtcRecordVideoLayoutMode_CUSTOM = 3,
};

/**
 * @brief 录制视频布局Region （仅适用于 Mac 系统）。
 */
__attribute__((visibility("default"))) @interface DingRtcRecordVideoRegion : NSObject

@property (nonatomic, strong) NSString * _Nonnull userId;
@property (nonatomic, assign) DingRtcVideoSource sourceType;

@end

/**
 * @brief 录制视频布局 （仅适用于 Mac 系统）。
 */
__attribute__((visibility("default"))) @interface DingRtcRecordVideoLayout : NSObject
/** 视频布局模式。 */
@property (nonatomic, assign) DingRtcRecordVideoLayoutMode mode;
/** 录制视频背景色。 */
@property (nonatomic, assign) DingRtcRecordVideoBgColor backColor;
/** 录制视频用户布局数组。 */
@property (nonatomic, strong) NSMutableArray<DingRtcRecordVideoRegion *> * _Nullable shapes;

@end

/**
 * @brief 音频分组成员信息
 */
__attribute__((visibility("default"))) @interface DingRtcAudioGroupMemberInfo : NSObject

/** 用户userId */
@property (nonatomic, strong) NSString * _Nonnull userId;
@property (nonatomic, strong) NSString * _Nullable userData;

@end

#if TARGET_OS_MAC && !TARGET_OS_IPHONE

/**
 * @brief 录制视频模板 （仅适用于 Mac 系统）。
 */
__attribute__((visibility("default"))) @interface DingRtcRecordTemplate : NSObject
/** 录制类型。 */
@property (nonatomic, assign) DingRtcRecordType recordType;
/** 录制格式。 */
@property (nonatomic, assign) DingRtcRecordFormat recordFormat;
/** 音频采样率类型。 */
@property (nonatomic, assign) DingRtcAudioSampleRate sampleRate;
/** 音频录制质量。 */
@property (nonatomic, assign) DingRtcAudioQuality audioQuality;
/** 外部pcm采集。 */
@property (nonatomic, assign) bool enableRecordExternalCapturePCM;
/** 外部pcm渲染。 */
@property (nonatomic, assign) bool enableRecordExternalRenderPCM;
/** 录制视频分辨率。 */
@property (nonatomic, assign) DingRtcRecordVideoCanvasConfig canvas;
/** 帧率。 */
@property (nonatomic, assign) int fps;
/** 码率。 */
@property (nonatomic, assign) int bitrate;
/** 分段录制。 */
@property (nonatomic, assign) bool isFragment;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 屏幕分享区域 （仅适用于 Mac 系统）。
 */
__attribute__((visibility("default"))) @interface DingRtcScreenShareRegion : NSObject

/** x轴起点 */
@property (nonatomic, assign) int originX;
/** y轴起点 */
@property (nonatomic, assign) int originY;
/** 宽度 */
@property (nonatomic, assign) int width;
/** 高度 */
@property (nonatomic, assign) int height;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 屏幕分享配置 （仅适用于 Mac 系统）。
 */
__attribute__((visibility("default"))) @interface DingRtcScreenShareConfig : NSObject

/** 屏幕分享时是否指定区域，默认为 NO */
@property (nonatomic, assign) BOOL isShareByRegion;
/** 屏幕分享区域配置 */
@property (nonatomic, strong, nullable) DingRtcScreenShareRegion *shareRegion;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 屏幕分享源信息 （仅适用于 Mac 系统）。
 */
__attribute__((visibility("default"))) @interface DingRtcScreenSourceInfo : NSObject

@property (nonatomic, copy) NSString * _Nonnull sourceId;
@property (nonatomic, copy) NSString * _Nonnull sourceName;
@property (nonatomic, strong) DingRtcVideoFrame * _Nonnull thumbnail;
@property (nonatomic, strong) DingRtcVideoFrame * _Nonnull icon;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 屏幕分享配置信息 （仅适用于 Mac 系统）。
 */
__attribute__((visibility("default"))) @interface DingRtcScreenShareInfo : NSObject

/** 屏幕分享源类型。 */
@property (nonatomic, assign) DingRtcScreenShareType sourceType;
/** 屏幕分享源ID。 */
@property (nonatomic, copy) NSString * _Nonnull sourceId;
/** 屏幕分享配置。 */
@property (nonatomic, strong) DingRtcScreenShareConfig * _Nonnull config;

@end

/**
 * @ingroup OC_DingRtcDef
 * @since 3.0
 * @remark mac
 * @brief 设备信息 （仅适用于 Mac 系统）。
 */
__attribute__((visibility("default"))) @interface DingRtcDeviceInfo : NSObject

/** 设备名称。 */
@property (nonatomic, copy) NSString * _Nullable deviceName;
/** 设备ID。 */
@property (nonatomic, copy) NSString * _Nullable deviceID;
/** 设备传输类型。 */
@property (nonatomic, assign) DingRtcDeviceTransportType deviceTransportType;

@end

#endif
